
> library(sp); library(rgdal)
> library(spdep); require(kohonen)
> library(RColorBrewer)

> # set workspace
> setwd("D:/workspace")
> getwd()
[1] "D:/workspace"
> # Download zip from the web
> #download.file("http://ADD_LINK/data_amsterdam.zip", "data_amsterdam.zip")
> # unzip file
> unzip("data_amsterdam.zip")

> # load shapefile
> data <- readOGR(".", "data_amsterdam")
OGR data source with driver: ESRI Shapefile 
Source: ".", layer: "data_amsterdam"
with 5242 features and 29 fields
Feature type: wkbPolygon with 2 dimensions
> # save(data,file="data_amsterdam")
> # projection of the data
> proj4string(data)
[1] "+proj=sterea +lat_0=52.15616055555555 +lon_0=5.38763888888889 +
k=0.9999079 +x_0=155000 +y_0=463000 +ellps=bessel +units=m +no_defs"
> # bounding box
> bbox(data)
       min      max
x 111969.8 130800.0
y 477900.0 493142.9

> # show attributes
> names(data)
[1] "DENS" "ID"   "PROX" "NATI" "HOUS" "VARI"
> # summary attributes
> summary(data[, c("NATI", "HOUS", "PROX", "DENS", "VARI")])
Object of class SpatialPolygonsDataFrame
Coordinates:
       min      max
x 111969.8 130800.0
y 477900.0 493142.9
Is projected: TRUE 
proj4string :
[+proj=sterea +lat_0=52.15616055555555 +lon_0=5.38763888888889 +k=0.9999079 +
x_0=155000 +y_0=463000 +ellps=bessel
+units=m +no_defs]
Data attributes:
      NATI            HOUS             PROX               DENS             VARI       
 Min.   :1.000   Min.   :  35.0   Min.   :   0.212   Min.   : 0.000   Min.   : 104.3  
 1st Qu.:1.000   1st Qu.: 176.0   1st Qu.: 349.360   1st Qu.: 1.000   1st Qu.: 707.3  
 Median :2.000   Median : 228.0   Median : 560.659   Median : 2.000   Median :1067.7  
 Mean   :2.275   Mean   : 273.9   Mean   : 684.490   Mean   : 2.654   Mean   :1240.4  
 3rd Qu.:3.000   3rd Qu.: 305.0   3rd Qu.: 891.356   3rd Qu.: 4.000   3rd Qu.:1592.7  
 Max.   :5.000   Max.   :2675.0   Max.   :7454.897   Max.   :10.000   Max.   :7525.8  
> str(data@data)
'data.frame':	5242 obs. of  6 variables:
 $ DENS: num  0 0 0 0 1 0 0 0 0 0 ...
 $ ID  : Factor w/ 5242 levels "E1119N4855","E1120N4854",..: 5103 5145 5158 ...
 $ PROX: num  4819 5019 5119 1314 1085 ...
 $ NATI: num  5 5 5 2 5 3 3 4 3 1 ...
 $ HOUS: num  523 438 249 210 343 293 241 265 247 236 ...
 $ VARI: num  5217 5417 5517 2412 2183 ...

> # select variables for som-based clustering
> data.som <- as.matrix(data@data[, c("NATI", "HOUS", "PROX", "DENS", "VARI")])
> # scale selected variables 
> data.som.sc <- scale(data.som)
> # create som topology
> som.grid <- somgrid(xdim = 10, ydim = 8, topo = c("hexagonal"))
> set.seed(20082014)
> # train som
> som.res <- som(data=data.som.sc, grid=som.grid, rlen=100, alpha = c(0.05, 0.01), 
+                keep.data=TRUE, n.hood="circular")
> summary(som.res)
som map of size 10x8 with a hexagonal topology.
Training data included; dimension is 5242 by 5
Mean distance to the closest unit in the map: 0.3151507

> # set colors
> coolBlueHotRed <- function(n, alpha = 1) { rainbow(n, end=4/6, alpha=alpha)[n:1] }
> # plot som training progress
> plot(som.res, type="changes")
> # plot how many cells are mapped to each neuron
> plot(som.res, type="counts", palette.name=coolBlueHotRed)
> # U-Matrix to explore clusters
> plot(som.res, type="dist.neighbours", palette.name=coolBlueHotRed)  
> # areas with large distances refer to boundaries between clusters
> colnames(som.res$code) # plots normalised component pl.
[1] "NATI" "HOUS" "PROX" "DENS" "VARI"
> plot(som.res, type = "property", property = som.res$codes[,"PROX"], 
+ main=c("Proximity"), palette.name=coolBlueHotRed) #
> plot(som.res, type = "property", property = som.res$codes[,"DENS"], 
+ main=c("Density"), palette.name=coolBlueHotRed)

> # cluster component planes through k-means and
> # determine number of clusters by means of minimizing the 
> # within cluster sum of squares
> som.cluster <- som.res$codes 
> wss <- (nrow(som.cluster) - 1) * sum(apply(som.cluster, 2, var)) 
> for (i in 2:10) {
+   set.seed(20082014)
+   wss[i] <- sum(kmeans(som.cluster, centers=i)$withinss)
+ }
> plot(wss, type="o", xlab="Number of clusters")
> # re-run k-means with 7 clusters
> set.seed(20082014)
> som.cluster.res <- kmeans(som.cluster, 7)$cluster

> # plot the clustered component planes
> plot(som.res, type="mapping", bgcol = brewer.pal(7, "Set1")[som.cluster.res], main = "Clusters") 
> add.cluster.boundaries(som.res, som.cluster.res)
> # plot the clustered component planes
> plot(som.res, type="mapping", bgcol = brewer.pal(7, "Set1")[som.cluster.res], main = "Clusters") 
> add.cluster.boundaries(som.res, som.cluster.res)

> # plot clusters on a geographical map
> cluster_details <- data.frame(id=data$ID, cluster=som.cluster.res[som.res$unit.classif])
> data@data <- data.frame(data@data, cluster_details[match(data@data[,"ID"], cluster_details[,"id"]), ])
> data$fcluster <- as.factor(data$cluster)
> spplot(data, "fcluster", col="transparent", col.regions=brewer.pal(7, "Set1"))
 
 







